﻿// ----------------------------------------------------------------------------------
// <copyright file="DefaultEntityMemberInfo`2.cs" company="NMemory Team">
//     Copyright (C) 2012-2013 NMemory Team
//
//     Permission is hereby granted, free of charge, to any person obtaining a copy
//     of this software and associated documentation files (the "Software"), to deal
//     in the Software without restriction, including without limitation the rights
//     to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
//     copies of the Software, and to permit persons to whom the Software is
//     furnished to do so, subject to the following conditions:
//
//     The above copyright notice and this permission notice shall be included in
//     all copies or substantial portions of the Software.
//
//     THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
//     IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
//     FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
//     AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
//     LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
//     OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
//     THE SOFTWARE.
// </copyright>
// ----------------------------------------------------------------------------------

namespace NMemory.Common
{
    using System;
    using System.Linq.Expressions;
    using System.Reflection;
    using NMemory.Exceptions;

    public class DefaultEntityMemberInfo<TEntity, TMember> : 
        IEntityMemberInfo<TEntity, TMember>,
        IEntityMemberInfoServicesProvider<TEntity, TMember>
    {
        private readonly MemberInfo memberInfo;

        private IEntityMemberInfoServices<TEntity, TMember> services;

        public DefaultEntityMemberInfo(MemberInfo member)
        {
            if (member == null)
            {
                throw new ArgumentNullException("member");
            }

            if (member.DeclaringType != typeof(TEntity))
            {
                throw new ArgumentException(ExceptionMessages.Missing, "member");
            }

            if (ReflectionHelper.GetMemberType(member) != typeof(TMember))
            {
                throw new ArgumentException(ExceptionMessages.Missing, "member");
            }

            this.memberInfo = member;
        }

        public DefaultEntityMemberInfo(Expression<Func<TEntity, TMember>> memberSelector)
        {
            if (memberSelector == null)
            {
                throw new ArgumentNullException("memberSelector");
            }

            MemberExpression member = memberSelector.Body as MemberExpression;

            if (member == null)
            {
                throw new ArgumentException(ExceptionMessages.Missing, "memberSelector");
            }

            this.memberInfo = member.Member;
        }

        public Type EntityType
        {
            get { return typeof(TEntity); }
        }

        public Type MemberType
        {
            get { return typeof(TMember); }
        }

        public MemberInfo Member
        {
            get { return this.memberInfo; }
        }

        public IEntityMemberInfoServices<TEntity, TMember> EntityMemberInfoServices
        {
            get 
            {
                if (this.services == null)
                {
                    this.InitializeEntityMemberInfoServices();
                }

                return this.services;
            }
        }

        public void InitializeEntityMemberInfoServices()
        {
            if (this.services == null)
            {
                this.services = new DefaultEntityMemberInfoServices<TEntity, TMember>(this);
            }
        }
    }
}
